import os
import logging
import numpy as np
import torch
from PIL import Image
import tqdm as tqdm

import torchvision
from torchvision.models.detection.faster_rcnn import FastRCNNPredictor
from torchvision.models.detection.mask_rcnn import MaskRCNNPredictor

from engine import train_one_epoch, evaluate
import utils
import transforms as T

from torch.utils.tensorboard import SummaryWriter
import argparse

class myTestDataset(object):
    def __init__(self, root, transforms):
        self.root = root
        self.transforms = transforms
        self.imgs = list(sorted(os.listdir(os.path.join(root, "images"))))
        self.masks = list(sorted(os.listdir(os.path.join(root, "images"))))

    def __getitem__(self, idx):
        img_path = os.path.join(self.root, "images", self.imgs[idx])
        mask_path = os.path.join(self.root, "images", self.masks[idx])
        img = Image.open(img_path).convert("RGB")


        target = {}

        if self.transforms is not None:
            img, target = self.transforms(img, target)

        return img, img_path 


    def __len__(self):
        return len(self.imgs)


def get_transform(train):
    transforms = []
    transforms.append(T.ToTensor())
    if train:
        transforms.append(T.RandomHorizontalFlip(0.5))
    return T.Compose(transforms)



def test_net(#net=net,
        device,
        epochs=1,
        batch_size=1,
        lr=0.005,
        img_scale=1.0,
        val_percent=0.1,
        test_path='test'):
    dataset_test = myTestDataset(test_path, get_transform(train=False))

    model = torch.load('model_inference_daphnia_magna_train600_epoch20.pth')

    params = [p for p in model.parameters() if p.requires_grad]
    optimizer = torch.optim.SGD(params, lr=lr,
                                momentum=0.9, weight_decay=0.0005)
    lr_scheduler = torch.optim.lr_scheduler.StepLR(optimizer,
                                                   step_size=3,
                                                   gamma=0.1)

    print("# test: ", len(dataset_test))
    model.eval()
    for i in range(len(dataset_test)):
        img, img_path = dataset_test[i]
        print(i,": ", img_path)
        save_path = img_path.replace('.png', '_result.png')

        with torch.no_grad():
            prediction = model([img.to(device)])

        img_pil = Image.fromarray(img.mul(255).permute(1, 2, 0).byte().numpy())

        result_np = prediction[0]['masks'][0, 0].mul(255).byte().cpu().numpy()

        result_np[result_np<127] = 0 
        result_np[result_np>=127] = 255 

        result_mask = Image.fromarray(result_np)
        result_mask.save(save_path)


    print("Done!")

def get_args():
    parser = argparse.ArgumentParser(description='Train the UNet on images and target masks',
                      formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-e', '--epochs', metavar='E', type=int, default=5,
                      help='Number of epochs', dest='epochs')
    parser.add_argument('-b', '--batch-size', metavar='B', type=int, nargs='?', default=1,
                      help='Batch size', dest='batchsize')
    parser.add_argument('-l', '--learning-rate', metavar='LR', type=float, nargs='?', default=0.0001,
                      help='Learning rate', dest='lr')
    parser.add_argument('-f', '--load', dest='load', type=str, default=False,
                      help='Load model from a .pth file')
    parser.add_argument('-s', '--scale', dest='scale', type=float, default=0.5,
                      help='Downscaling factor of the images')
    parser.add_argument('-v', '--validation', dest='val', type=float, default=10.0,
                      help='Percent of the data that is used as validation (0-100)')
    parser.add_argument('-t', '--test-path', dest='test_path', type=str, default='test',
                      help='Directory of test dataset')
    return parser.parse_args()

    
if __name__ == "__main__":
    logging.basicConfig(filename='myLog.txt', level=logging.INFO, format='%(levelname)s: %(message)s')

    args = get_args()

    device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
    logging.info(f'Using device {device}')



try:
    test_net(#net=net,
        epochs=args.epochs,
        batch_size=args.batchsize,
        lr=args.lr,
        device=device,
        img_scale=args.scale,
        val_percent=args.val / 100,
        test_path = args.test_path)
except KeyboardInterrupt:
    torch.save(net.state_dict(), 'INTERRUPTED.pth')
    logging.info('Saved interrupt')
    try:
        sys.exit(0)
    except SystemExit:
        os._exit(0)

